<?php

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

 namespace Civi\FormProcessor\OutputHandler;

 use \Civi\FormProcessor\Config\ConfigurationBag;
 use \Civi\FormProcessor\Config\Specification;
 use \Civi\FormProcessor\Config\SpecificationBag;
 use \Civi\FormProcessor\Config\SpecificationCollection;
 use \Civi\FormProcessor\Config\SpecificationFields;
 use \Civi\FormProcessor\OutputHandler\AbstractOutputHandler;
 use \Civi\FormProcessor\DataBag;

 use CRM_FormProcessor_ExtensionUtil as E;

 class FormatOutput extends AbstractOutputHandler {

  /**
   * Get the configuration specification
   *
   * @return SpecificationBag
   */
  public function getConfigurationSpecification() {
    return new SpecificationBag(array(
      new SpecificationCollection(
        'fields',
        E::ts('Output fields'),
        new SpecificationBag(array(
          new SpecificationFields('field', E::ts('Field'), true),
          new Specification('output_name', 'String', E::ts('Output name'), true, null, null, null),
        ))
      )
    ));
  }

  /**
   * Convert the action data to output data.
   *
   * @param DataBag $dataBag
   * @return array
   */
  public function handle(DataBag $dataBag) {
    $output = array();
    $fields = $this->configuration->get('fields');
    foreach($fields as $field) {
      $output[$field['output_name']] = $dataBag->getDataByAlias($field['field']);
    }
    return $output;
  }

  /**
   * Returns the label of the output handler.
   *
   * @return string
   */
  public function getLabel() {
    return E::ts('Decide what to send');
  }

 }
