<?php

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

 namespace Civi\FormProcessor\Validation;

 use \Civi\FormProcessor\Validation\AbstractValidator;
 use \Civi\FormProcessor\Validation\EmailValidator;

 use CRM_FormProcessor_ExtensionUtil as E;

 class Factory {

  /**
   * @var array<AbstractValidator>
   */
  protected $validators = array();

  /**
   * @var Factory
   */
  private static $singleton;

  private function __construct() {
    $this->addValidator(new EmailValidator());
    $this->addValidator(new MinLengthValidator());
    $this->addValidator(new MinValidator());
    $this->addValidator(new MaxValidator());
    $this->addValidator(new DateValidator());
  }

  /**
   * @return Factory
   */
  public static function singleton(){
    if (!self::$singleton) {
      self::$singleton = new Factory();
    }
    return self::$singleton;
  }

  /**
   * Adds a validator
   *
   * @param AbstractFactory $validator
   * @return Factory
   */
  public function addValidator(AbstractValidator $validator) {
    $this->validators[$validator->getName()] = $validator;
    return $this;
  }

  /**
   * Returns a validator
   *
   * @param string $name
   *   The name of the validator.
   * @return AbstractValidator
   */
  public function getValidatorByName($name) {
    return $this->validators[$name];
  }

  /**
   * @return array<AbstractValidator>
   */
  public function getValidators() {
    return $this->validators;
  }
 }
