(function(angular, $, _) {
  // Declare a list of dependencies.
  angular.module('form_processor', [
    'crmUi', 'crmUtil', 'ngRoute', 'action_provider'
  ]);
  angular.module('form_processor').factory('formProcessorFactory', ["crmApi", "$q", "actionProviderFactory", function(crmApi, $q, actionProviderFactory) {
    var actionTypes = {};

    // Initialize the context.
    var setContext = function(context) {
			if (!(context in actionTypes)) {
				actionTypes[context] = {};
			}
		};

		var setAction = function(type, context, action) {
			setContext(context);
			actionTypes[context][type] = action;
			actionProviderFactory.setAction(type, context, action);
		};

  	return {
			getAction: function(type, context) {
				return actionProviderFactory.getAction(type, context);
			},

  		getFieldList: function (inputs, actions, selectedAction, context) {
				setContext(context);

  		  var defer = $q.defer();
	  		var fields = [];
	  		var promises = [];
	  		angular.forEach(inputs, function(input, key) {
  				var field = {
						'id': 'input.'+input.name,
						'text': 'Input::'+input.name,
  					'name': 'input.'+input.name,
  					'label': 'Input::'+input.name
  				};
  				fields.push(field);
				});

				var skipAfter = false;
				var actionApiCalls = {};
				angular.forEach(actions, function(action, key) {
					if (selectedAction && selectedAction.name == action.name) {
						skipAfter = true;
					}
					if (!skipAfter && !action.delay) {
						if (!(action.type in actionTypes[context])) {
							actionApiCalls[key] = [
									'ActionProvider',
									'getaction',
									{"name": action.type, "context":context}
							];
						} else {
							angular.forEach(actionTypes[context][action.type].output_spec, function(output_field, output_field_key) {
								var field = {
									'id': 'action.'+action.name+'.'+output_field.name,
									'text': 'Action::'+action.title+'::'+output_field.title,
									'name': 'action.'+action.name+'.'+output_field.name,
									'label': 'Action::'+action.title+'::'+output_field.title
								};
								fields.push(field);
							});
						}
					}
				});
				if (Object.keys(actionApiCalls).length) {
					promises.push(
						CRM.api3(actionApiCalls)
						.then(function(results) {
							angular.forEach(results, function(actionType, key) {
								var action = actions[key];
								setAction(action.type, context, actionType);
								angular.forEach(actionType.output_spec, function(output_field, output_field_key) {
									var field = {
										'id': 'action.' + action.name + '.' + output_field.name,
										'text': 'Action::' + action.title + '::' + output_field.title,
										'name': 'action.' + action.name + '.' + output_field.name,
										'label': 'Action::' + action.title + '::' + output_field.title
									};
									fields.push(field);
								});
							});
							return $q.resolve();
						})
					);
				}

				$q.all(promises).then(function() {
				  defer.resolve(fields);
				});

				return defer.promise;
  		}
		};
  }]);
})(angular, CRM.$, CRM._);
