(function(angular, $, _) {

  angular.module('form_processor').config(function($routeProvider) {
      $routeProvider.when('/formProcessor/new', {
        templateUrl: '~/form_processor/FormProcessorEditCtrl.html',
        controller: 'FormProcessorEditCtrl',
        resolve: {
        	apiCalls: function($route, crmApi) {
        		reqs = {};
        		reqs.formProcessor = {
          		name: '',
          		title: '',
          		is_active: '1',
          		inputs: [],
          		actions: [],
          		output_handler: false,
          		output_handler_configuration: [],
          		enable_default_data: '0',
          		default_data_output_configuration: {},
          		default_data_inputs: [],
          		default_data_actions: [],
        		};
          	return reqs;
          }
        }
      });
      $routeProvider.when('/formProcessor/:id', {
        templateUrl: '~/form_processor/FormProcessorEditCtrl.html',
        controller: 'FormProcessorEditCtrl',
        resolve: {
        	apiCalls: function($route, crmApi) {
        		var reqs = {};
            if ($route.current.params.id !== 'new') {
              reqs.formProcessor = ['FormProcessorInstance', 'getsingle', {
                id: $route.current.params.id
              }];
            }
            return crmApi(reqs);
         	}
        }
      });
    }
  );

  angular.module('form_processor').controller('FormProcessorEditCtrl', function($scope, formProcessorFactory, dialogService, crmApi, apiCalls, $location, $window) {
  	var ts = $scope.ts = CRM.ts(null);

    $scope.formProcessor = apiCalls.formProcessor;
    $scope.locks = {name: true};
    $scope.isNameValid = false;
    $scope.new_id = -1;
    $scope.deletedInputs = [];
    $scope.deletedActions = [];
    $scope.deletedDefaultDataInputs = [];
    $scope.deletedDefaultDataActions = [];
    $scope.default_data_fields = { results: []}
    $scope.permissions = CRM.form_processor.permissions;
    $scope.actionTitles = CRM.form_processor.actionTitles;
    $scope.defaultDataActionTitles = CRM.form_processor.defaultDataActionTitles;

    $scope.outputHandlers = CRM.form_processor.outputHandlers;
    if (!$scope.formProcessor.output_handler) {
    	$scope.formProcessor.output_handler = CRM.form_processor.defaultOutputHandler;
    }

    // Convert to an object. Otherwise saving the configuration will fail because of a mix up between
    // object and array type.
    if ($scope.formProcessor.default_data_output_configuration instanceof Array) {
    	$scope.formProcessor.default_data_output_configuration = {};
    }

    $scope.$watch('formProcessor.name', function(newFormProcessorName, oldFrmProcessorName) {
    	// Watch for changes in the name field
      crmApi('FormProcessorInstance', 'validatename', {'name': newFormProcessorName,'id': $scope.formProcessor.id})
      .then(function(data) {
      	$scope.isNameValid = false;
      	if (data.is_valid) {
      		$scope.isNameValid = true;
      	}
      });
    }, true);

    $scope.$watch('formProcessor.title', function(newFormProcessorTitle, oldFrmProcessorTitle) {
    	// Watch for changes in the name field
    	if ($scope.locks.name && !$scope.formProcessor.id) {
    		$scope.formProcessor.name = newFormProcessorTitle;
    		$scope.formProcessor.name = $scope.formProcessor.name.toLowerCase();
    		$scope.formProcessor.name = $scope.formProcessor.name.replace(/[^a-zA-Z\d]/g, '_');
    		$scope.formProcessor.name = $scope.formProcessor.name.replace(/_+/g, '_');
    	}
    }, true);

    // Watch for changes in the default data actions if changed we need to populate the available fields again.
    $scope.$watchCollection('formProcessor.default_data_actions', function (newActions, oldActions) {
    	 formProcessorFactory.getFieldList([], $scope.formProcessor.default_data_actions, false, "form_processor_default_data")
    	 .then(function(fields) {
         $scope.default_data_fields.results = fields;
         $("input.defaultDataOutputSelect2").trigger('change.select2');
    	 });
    });

    $scope.isValidName = function(name) {
    	return (!name) || name.match(/^[a-z0-9_]+$/) ? true : false;
    };

    $scope.save = function(goBack) {
    	var output_handler = angular.copy($scope.formProcessor.output_handler);
    	$scope.formProcessor.output_handler = output_handler.name;
    	$scope.formProcessor.output_handler_configuration = output_handler.configuration;
      var result = crmApi('FormProcessorInstance', 'create', $scope.formProcessor, true);
      result.then(function(data) {
        var apiCalls = {};
        $scope.formProcessor.id = data.id;
        $scope.formProcessor.output_handler = output_handler;

        apiCalls = $scope.createSaveInputApiCalls(apiCalls, $scope.formProcessor.inputs, $scope.deletedInputs, 'FormProcessorInput', $scope.formProcessor);
        apiCalls = $scope.createSaveInputApiCalls(apiCalls, $scope.formProcessor.default_data_inputs, $scope.deletedDefaultDataInputs, 'FormProcessorDefaultDataInput', $scope.formProcessor);
        apiCalls = $scope.createSaveActionApiCalls(apiCalls, $scope.formProcessor.actions, $scope.deletedActions, 'FormProcessorAction', $scope.formProcessor);
        apiCalls = $scope.createSaveActionApiCalls(apiCalls, $scope.formProcessor.default_data_actions, $scope.deletedDefaultDataActions, 'FormProcessorDefaultDataAction', $scope.formProcessor);
        CRM.api3(apiCalls)
        .then(function(results) {
          var apiCalls2 = {};
          angular.forEach(results, function(result, key) {
            var index;
            if (key.startsWith('create_FormProcessorInput_')) {
              index =  key.substr(26);
              $scope.formProcessor.inputs[index].id = result.id;
              apiCalls2 = $scope.createSaveValidatorApiCalls(apiCalls2, $scope.formProcessor.inputs[index], 'FormProcessorInput');
            }
            if (key.startsWith('create_FormProcessorDefaultDataInput_')) {
              index =  key.substr(37);
              $scope.formProcessor.default_data_inputs[index].id = result.id;
              apiCalls2 = $scope.createSaveValidatorApiCalls(apiCalls2, $scope.formProcessor.inputs[index], 'FormProcessorDefaultDataInput');
            }
            if (key.startsWith('create_FormProcessorAction_')) {
              index =  key.substr(27);
              $scope.formProcessor.actions[index].id = result.id;
            }
            if (key.startsWith('create_FormProcessorDefaultDataAction_')) {
              index =  key.substr(38);
              $scope.formProcessor.default_data_actions[index].id = result.id;
            }
          });

          CRM.api3(apiCalls2)
          .then(function(results) {
            if (goBack) {
              $scope.goback();
            } else {
              $scope.editFormProcessorForm.$setPristine();
            }
          });
        });
      });
    };

    $scope.goback = function () {
      $location.path('/formprocessors');
      $window.location.href = $location.absUrl();
    };

    /**
     * Create a set of api calls to save the validators.
     *
     * @param apiCalls
     * @param input
     * @param api
     * @returns {*}
     */
    $scope.createSaveValidatorApiCalls = function(apiCalls, input, api) {
      if (input.deletedValidators) {
        angular.forEach(input.deletedValidators, function (deletedValidator, validator_index) {
          if (deletedValidator.id) {
            apiCalls['delete_FormProcessorValidation_'+deletedValidator.id] = ['FormProcessorValidation', 'delete', {'id': deletedValidator.id}];
          }
        });
      }

      angular.forEach(input.validators, function(validator, validator_index){
        validator.entity = api;
        validator.entity_id = input.id;
        var validatorParams = angular.copy(validator);
        validatorParams.validator = validator.validator.name;
        apiCalls[api+'_FormProcessorValidation_'+input.id+'_'+validator_index] = ['FormProcessorValidation', 'create', validatorParams];
      });
      return apiCalls;
    };

    /**
     * Create a set of api calls to save all the inputs.
     *
     * @param apiCalls
     * @param inputs
     * @param deletedInputs
     * @param api
     * @param formProcessor
     * @returns {*}
     */
    $scope.createSaveInputApiCalls = function(apiCalls, inputs, deletedInputs, api, formProcessor) {
    	angular.forEach(deletedInputs, function(input, key) {
      	if (input.id) {
      	  apiCalls['delete_'+api+'_'+key] = [api, 'delete', {'id': input.id}];
      	}
      });

      angular.forEach(inputs, function(input, key) {
      	if (input.id < 1) {
      		delete input.id;
      	}
      	input.form_processor_instance_id = formProcessor.id;
        var inputParam = angular.copy(input);
        inputParam.type = input.type.name;
      	apiCalls['create_'+api+'_'+key] = [api, 'create', inputParam];
      });
      return apiCalls;
    };

    /**
     * Create a set of api Calls to save all the actions..
     *
     * @param apiCalls
     * @param actions
     * @param deletedActions
     * @param api
     * @param formProcessor
     * @returns {*}
     */
    $scope.createSaveActionApiCalls = function(apiCalls, actions, deletedActions, api, formProcessor) {
    	angular.forEach(deletedActions, function(action, key) {
      	if (action.id) {
          apiCalls['delete_'+api+'_'+key] = [api, 'delete', {'id': action.id}];
      	}
      });

      var actionWeight = 0;
      angular.forEach(actions, function(action, key) {
      	var old_id = action.id;
      	if (action.id < 0) {
      		delete action.id;
      	}
      	action.form_processor_instance_id = formProcessor.id;
      	action.weight = actionWeight;
      	if (action.delay === null) {
      	  action.delay = '';
      	  action.delay_configuration = '';
        }
        apiCalls['create_'+api+'_'+key] = [api, 'create', action];
      	actionWeight++;
      });
      return apiCalls;
    };

    $scope.editOutputHandler = function editOutputHandler() {
    	var options = CRM.utils.adjustDialogDefaults({
        autoOpen: false,
        width: '40%',
        height: 'auto',
        title: ts('Edit output handler')
      });

      formProcessorFactory.getFieldList($scope.formProcessor.inputs, $scope.formProcessor.actions, false, 'form_processor')
      .then(function (fields) {
        var model = {
      	 formProcessor: $scope.formProcessor,
      	 outputHandler: $scope.formProcessor.output_handler,
      	 outputHandlers: $scope.outputHandlers,
      	 fields: fields
        };
        dialogService.open('OutputHandlerDialog', '~/form_processor/OutputHandlerDialogCtrl.html', model, options)
        .then(function(data) {
      	  $scope.editFormProcessorForm.$setDirty();
      	  $scope.formProcessor.output_handler = data.outputHandler;
			  });
		  });
    };
  });

})(angular, CRM.$, CRM._);
